c
c *********************************************************************
c
c                      -------------------
c                     |      MAGRAV2      |
c                      -------------------
c
c          INTERACTIVE GRAVITY AND MAGNETICS MODELLING
c
c           Revision 2.1 ; edited Sept. 24, 1992
c
c ** NOTE: Source code for this program is in 5 files :
c          magrav2.for, ms1.for, ms2.for, ms3.for, ms4.for,
c          ms5.for, ms6.for, and ms7.for
c          File: "magrav.cmn" is included in all source files.
c
c          Libraries for compilation :
c          Microsoft FORTRAN (3.31 to 5.0), Halo Professional (2.0)
c          and assembler object file :sound.
c
c ********************************************************************
c
c  This program is an improved version of MAGRAV, the history of 
c      which follows:
c
c  Author: I. Wells, Computing Services ,Bedford institute of
c  Oceanography(BIO). for Dr. Haworth, Atlantic Geoscience Centre.
c
c Reference : 
c  Wells, I.(1979) MAGRAV - A computer program to create two
c   dimensional gravity and/or magnetic models,
c   Computer Science Centre OPEN FILE 597.
c
c  Magnetics modified by Franca M Lindia, August 1982.
c  Modifications based on 2.5-D magnetic equations of
c  Shuey and Pasquale (1973).
c
c  Gravity equations modified by Peter McGrath, December 1982,
c  for 2.5 D bodies. See Rasmussen and Pederson (1979)
c  Geophys. Prosp., 27, 749-760.
c  Other minor modifications to the program were also made.
c
c  Modified by Peter McGrath, December 1982, to permit
c  automatic adjustment of body magnetization (density)
c  contrasts and point movement using a non-linear
c  least squares algorithm published by Powell (1965) in
c  The Computer Journal, 7,p 303.
c
c  Rewritten in Microsoft FORTRAN 77 for use on an IBM-PC with
c  a raster graphics display driven by the HALO graphic system
c  by : John Broome, Potential Fields Section, 
c  Lithosphere and Canadian Shield Division, Geological Survey
c  of Canada, June 1986.
c __________________________________________________________________
c  Input/output:
c  =============
c
c Input : Input from keyboard or control from digitizer pad.
c Recovery file : "magrav.rec" used to allow you to go back
c           to a previous step.
c Models file : Contains named model data so you can terminate
c       a modeling session and then continue later.
c _________________________________________________________________
c  Variables in common blocks (Magrav.cmn) :
c  =========================================
c
c  ANOMAX(NTYPES): Maximum of anomaly window for plotting observed
c     and calculated anomalies.
c  ANOMIN(NTYPES): Minimum of anomaly window for plotting observed
c     and calculated anomalies.
c  BDEC(MAXBOD): Magnetization declination of each body. (mag only)
c  BDIP(MAXBOD): Magnetization dip for each body. (Mag only)
c  BDY(MAXBOD): Half-strike length of each body in km.
c  CALC(MAXBOD,MAXOBS): Anomaly calculated for each body,
c     at each observed point,CALC(MAXBOD+1,MAXOBS) stores the
c     combined anomaly from all bodies.
c  CMODNAM : name of current models file
c  DEC: Declination of field. (Mag only)
c       0 - 360. Degrees clockwise from North.
c  DIFMAX(NTYPES): Maximum of anomaly plotting window for difference
c        plots.
c  DIFMIN(NTYPES): Minimum of anomaly plotting window for difference
c        plots.
c  DIP: Dip of the field. (Mag only)
c       0 - 360. Degrees
c  E(22): Array used by DFS001
c  F(MAXOBS): Array used by DFS001 to store the calculated anomaly
c          values.
c  IANOM(MAXBOD): 0 Anomaly has not been calculated for this body
c                -1 Anomaly has been calculated for this body
c                   and is stored in calc
c  IB(4): Used by subroutine AMOVE to store body numbers for cases
c         where a point is found in more than 1 body.
c  IBR: Used in automatic optimization, set=1 for point movement
c          set=0 for contrast optimization.
c  IDELAY: factor used to control digitizer response time
c  IDIFF: -1 for observed and calculated plot
c          1 for difference between observed and calculated
c  IGRAP  0 = no graphics enabled
c         1 = VGA graphics
c         2 = NEC-MGE graphics
c  ILOC: flag to indicate locator configuration
c  IMOVES: starting profile point for profile segment fitting
c  IMOVEE: end profile point for profile segment fitting
c  IPRNT: flag to indicate printer configuration
c  IRECOV(NBACK): set to "1" if that rec. no. has been saved
c  ITYPE: 1 for gravity mode modelling.
c         2 for magnetics mode modelling.
c  JF: Digitizer tablet button status(4=pushed)
c  JX:     "       "      X   coordinate
c  JZ:     "       "      Z       "
c  KALK: Set=1 if anything changes that requires anomaly recalculation
c  KOMMNT: comments about model, 72 char. max.
c  LUNMOD: L.U.N. of models file
c  LUNCNF: L.U.N. of configuration file
c  LUNCOL: L.U.N  of colours file 
c  LUNSCR: L.U.N. of recovery file
c  MAXBOD: Maximum allowable number of bodies
c  MAXCAL: Maximum allowable number of bodies + 1.
c  MAXNPT: Maximum allowable number of points per body
c  MAXOBS: Maximum allowable number of observations
c  MODE: Set=1 - for full profile plotting(draw mode)
c        Set=2 - for plotting of only the "zoomed" window
c  MODEL: Unit number of model file
c  NAME: Name of current model entered with "name" option
c  NBACK; Current record number in recovery file.
c  NBODS: Number of bodies in current model
c  NFIELD(NTYPES): number of profile points for each modelling mode
c  NMOD: Current number of defined models in models file.
c  NP(4): Used by subroutine AMOVE to store point numbers in cases
c         where a point to be moved occurs in more than 1 body.
c  NPTS(MAXBOD): Number of points currently in each body
c  NSAV : number of steps saved in recovery file
c  NTYPES: Number of model types this program will handle. (Mag,Grav)
c  OBS(NTYPES,MAXOBS): Observed values for each mode and profile pt.
c  OFFSET(NTYPES): Offset added to observed for plotting only
c  OEND(NTYPES): last defined observed profile point
c  OSTART(NTYPES): first defined observed profile pointc  PI: Pi constant
c  RHOMAG(MAXBOD,ITYPE) Density(ITYPE=1), or Magnetization(ITYPE=2)
c        contrast for each body.
c  RMMAX(MAXBOD,NTYPES): Maximum value allowed for body contrast.
c  RMMIN(MAXBOD,NTYPES): Minimum value allowed for body contrast.
c  SKXMAX: Current right side sketch mode plotting limit
c  SKXMIN: Current left side sketch mode plotting limit.
c  SKZMAX: Current maximum depth for sketch mode plotting.
c  SKZMIN: Current minimum depth for sketch mode plotting.
c  SPACE(NTYPES): Spacing between field points
c  W(1275): Scratch work array used by DFS001, etc.
c  X(MAXNPT,MAXBOD): X Coordinates for each point in each body.
c  XC: Current X cursor position scaled to the current window.
c  XDIS: Distance in scaled units in X direction for resolution
c        on pinpointing for cursor input of bodies
c  XLEN(NTYPES): Length of profil in Km.
c  XLPL: Current right X plotting window limit
c  XMAX: Position in km of the last profile point.
c  XOFFS(NTYPES): X Offset of first profile reading in km.
c  XPOS(MAXNPT): X coordinate of each observed reading on profil
c  XTON: X to N angle. Orientation of X axis. (Mag only)
c      measured clockwise from North in degrees.
c  XUPL: Current left plotting window limit
c  XX(11): Array used by DFSOO1 to store variables being modified
c         for the best fit.
c  Z(MAXNPT,MAXBOD): Z Coordinates of each body point
c        Note - (0,0) is not an acceptible body point
c  ZC: Current Z cursor position scaled to the current window.
c  ZCON(ITYPE): A constant added to Z coord of all bodies
c  ZDIS: Allowable distance between the cursor and body point
c        positions in the Z direction.
c  ZLPL: Current bottom plotting window limit
c  ZMAX: Maximum distance from surface(0.) to bottom of screen
c  ZUPL: Current  top plotting window limit
c
c ****************************************************************
c
      PROGRAM MAGRAV2
c
      character*4 cans
      logical*2 fex
c
$include:'magrav.cmn'
c $nofloatcalls
c
      call ansist
      write(*,*) ed
      
      write(*,1111) sgr1
      write(*,299)
299   format('        MAGRAV2  2.1  (Sept. 24, 1992)',/,
     +   1h ,'        Halo Professional Version',/,
     +   1h ,'        ------------------------',//)
      write(*,1111) sgr0
      write(*,301)
301   format('  This interactive gravity and magnetic modelling',
     +' program is',/,' documented in :',//,' Broome, J.; 1986',/,
     +'   MAGRAV2: An Interactive Gravity and Magnetics Program',
     +' for',/,'   IBM-Compatible Microcomputers; Geological',
     +' Survey of Canada',/,'   Open File 1334, 83 p.',//,
     +' Broome, J.; 1988,',/,'   An IBM-Compatible Microcomputer',
     +' Workstation for Modelling and',/,'   Imaging Potential',
     +' Field Data; Computers and Geosciences, Vol. 14,',/,
     +'   No. 5, pp. 659-666.',/)
c
c Initialize program variables
c
      ccolnec = 'magnec.col'
      ccolvga = 'magvga.col'
      cmodnam = 'none'
      dec = 0.
      dip = 90.
      idelay = 5000
      idiff = -1
      itype = -9999
      kalk = 1
      lunscr = 9
      lunmod = 6
      luncnf = 15
      luncol = 16
      maxbod = 20
      maxcal = maxbod + 1
      maxnpt = 40
      maxobs = 500
      mode = 1
      nback = 0
      nbods = 0
      ntypes = 2
      skxmax = 0
      skxmin = 0.
      skzmin = 0
      skzmin = 0.
      xdis = 0.
      xlpl = 0.
      xton = 0.
      xupl = 0.
      zcon(1) = .001
      zcon(2) = zcon(1)
      zdis = 0.
      zlpl = 0.
      zmax = -1.
      zupl = 0.
c
      do 250 j = 1, ntypes
        offset(j) = 0.0
        xoffs(j) = 0.0
        xlen(j) = 0.0
        anomax(j) = 100.0
        anomin(j) = -100.0
        difmin(j) = -50.0
        difmax(j) = 50.0
        nfield(j) = 0
        ostart(j) = 0.0
        oend(j)   = 0.0
        space(j) = 1.0
        do 255 i = 1, maxobs
          obs(j,i) = 0.
255     continue
250   continue
c
      call rean
c
      do 260 i = 1 , maxbod
        npts(i) = 0
        rhomag(i,1) = 0.
        rhomag(i,2) = 0.
        rmmin(i,1) = 0.
        rmmin(i,2) = 0.
        rmmax(i,1) = 0.
        rmmax(i,2) = 0.
        bdec(i) = 0.
        bdip(i) = 0.
        bdy(i) = 100000
        do 265 j=1 , maxnpt + 1
          x(j,i) = 0.0
          z(j,i) = 0.0
265     continue
260   continue
c
c        kommnt ='                                                  '
c     +  //'                       '
c
c Open recovery file
c
      inquire(file='magrav.rec',exist=fex)
      if(fex.eqv..true.) then
        open(lunscr,file='magrav.rec',access='direct',status='old',
     +  form='unformatted',recl=65000)
      else
        open(lunscr,file='magrav.rec',access='direct',status='new',
     +  form='unformatted',recl=65000)
      endif
c
c Configure program based on "MAGRAV.CNF"
c
      call config(0)
c
c Select models file
c
      model = -99
c
      write(*,'(/a/)')' Type "HELP<ret>" for program information' 
      go to 400
c
c Branch to chosen option from value returned by "what"
c =====================================================
c
350   call sav
c
400   write(*,1111) sgr1
      write(*,'(/a\)')' Enter option : '
      write(*,1111) sgr0
      call sound(20,200)
      call what(iwhat)
      if(iwhat.le.0) then
499     write(*,*) 'pardon ?'
        goto 400
      endif
c
      if(itype.lt.0) then 
        if(iwhat.eq.24.or.iwhat.eq.25.or.iwhat.eq.8) go to 500
        if(iwhat.eq.34.or.iwhat.eq.29.or.iwhat.eq.18) go to 500
        if(iwhat.eq.15.or.iwhat.eq.26) go to 500
        if(iwhat.eq.39.or.iwhat.eq.38) go to 500
        write(*,*)'The modelling mode must be selected with'
        write(*,*)'options "GRAV"ity or "MAGN"etics, or a '
        write(*,*)'model must be "READ" in before this option'
        write(*,*)'can be called.'
        go to 400
      endif
500   goto(1000, 2000, 3000, 4000, 5000, 6000, 7000, 8000,
     +     9000, 10000, 11000, 12000, 13000, 14000, 15000,
     +    16000, 17000, 18000, 19000, 20000, 21000, 22000,
     +    23000, 24000, 25000, 26000, 27000, 28000, 29000,
     +    30000, 31000, 32000, 33000, 34000, 35000, 36000,
     +    37000, 38000, 39000)
     +    iwhat
c
c <EOBS> Enter or read in observed data
c -----------------------------------
1000  write(*,1111) ed
      if(itype.eq.1) write(*,*) 'You are in  GRAVITY MODE'
      if(itype.eq.2) write(*,*) 'You are in MAGNETICS MODE'
1014  write(*,'(a\)')' Is this the correct mode (y/n) : '
      read(*,'(a)',err=99000) cans
      if(cans.eq.'y'.or.cans.eq.'Y') then
        write(*,*)' '
        write(*,*)' 1 - Read observed data from a profile file'
        write(*,*)' 2 - Enter observed data manually'
        write(*,'(/a\)')' Select type of data input(1 or 2): '
        read(*,*,err=99000) intype
        if(intype.lt.1.or.intype.gt.2) go to 99000
        if(intype.eq.1)then
          call robse(ierr)
          if(ierr.eq.0) then
            go to 350
          else
            go to 400
          endif
        else
          call eobse
          go to 350
        endif
c
c Locate start observed profile point by skipping
c  "-9999.0" values
c
        do 1050 im=1, nfield(itype)
          if (obs(itype,im).ne.-9999.0) then
            ostart(itype) = im
            go to 1051
          endif
1050    continue
c
c Locate end observed profile point by locating
c  first trailing "-9999.0" value
c
1051    do 1060 im=ostart(itype), nfield(itype)
          if (obs(itype,im).eq.-9999.0) then
            oend(itype) = im-1
            go to 400
          else
            if(im.eq.nfield(itype)) oend(itype) = nfield(itype)
          endif
1060    continue
      else
        if(itype.eq.1)write(*,*)'Select option "MAGN"etics'
        if(itype.eq.2)write(*,*)'Select option "GRAV"ity'
        go to 400
      endif
c
c <EBOD> Input Body
c ---------------
2000  write(*,1111) ed
      do 2005 ibod=1,maxbod
        if(npts(ibod).eq.0) go to 2010
2005  continue
      call error('The maximum number of bodies exist')
      call bold ('To enter a new one, one must be deleted.')
      go to 400
c
2010  write(*,*)' Body',ibod,' will be created.'
2011  write(*,'(a\)')' Enter no. of points in body : '
      read(*,*,err=99000) npt
      if(npt.gt.maxnpt.or.npt.le.2) then
        write(*,*)'No. of points must be from 3 to ',maxnpt
        go to 2011
      endif
      npt = npt + 1
      npts(ibod) = npt
      npt1 = npt - 1
      write(*,*)'Enter ',npt1,' X and Z body point coord. pairs(km);'
      write(*,1111) sgr1
      write(*,*)'Note: points must be entered in clockwise order'
      write(*,1111) sgr0
      write(*,*)' '
      do 2173 i=1, npt1
        write(*,2150) i
2150    format(1h ,'Point ',i3,' X, Z : ',\)
        read(*,*,err=2175) x(i,ibod),z(i,ibod)
2173  continue
      x(npt,ibod) = x(1,ibod)
      z(npt,ibod) = z(1,ibod)
      nbods = nbods + 1
      kalk = 1
      if(igrap.ne.0) call plbod(0,igry)
      goto 350
2175  call error('Input error, new body deleted.')
      npts(ibod) = 0
      go to 400
c
c <MPOI> Move point
c ----------
3000  call poin
      goto 350
c
c <EPAR> Enter body parameters
c ---------------------
4000  write(*,1111) ed
      write(*,'(/a\)')' Enter body number for parameter change : '
      call sound(20,200)
      read(*,*,err=99000) ibod
      if(npts(ibod).eq.0) then
        call error('Body specified is not defined.')
        go to 400
      else
        call param(ibod)
        go to 350
      endif
c
c <CONT> Optimize contrast
c -----------------
5000  write(*,1111) ed
      write(*,'(/a)')' Enter body number for contrast optimization, '
      write(*,'(a\)')'  ( "0" for all bodies ) : '
      call sound (20,200)
      read(*,*,err=99000) ibod
      if(ibod.eq.0) then
        call ampl
        call planom(0,nclr)
        go to 350
      else
        if(npts(ibod).eq.0) then
          call error('Body specified is not defined.')
          go to 400
        else
          call imoves = 1
          call imovee = nfield(itype)
          call ampbod (ibod)
          call planom (0,nclr)
          go to 350
        endif
      endif 
      goto 350
c
c <DRAW> Draw graphics (full view)
c ----------------------
6000  if(nfield(itype).eq.0) go to 98000
      mode = 1
      xlpl = xpos(1,itype)
      xupl = xpos(nfield(itype),itype)
      zlpl = zmax
      zupl = 0
      if (igrap.ne.0) then
        call planom (0,iblk)
        if(idiff.lt.0) call plobs
        call plbod (0,igry)
      else
        call bold(' Graphics not enabled.')
      endif
      goto 400
c
c <SKET> Sketch mode (Draw area specified by zoom limits)
c -----------------------------------------------
7000  if(skxmin.eq.skxmax) then
        call bold (' Zoom not specified, call option "ZOOM"')
        go to 400
      endif
      if(nfield(itype).eq.0) go to 98000
      mode = 2
      xlpl = skxmin
      xupl = skxmax
      zlpl = skzmax
      zupl = skzmin
      if(igrap.ne.0) then
        call planom (0,iblk)
        if(idiff.lt.0)call plobs
        call plbod (0,igry)
      else
          call bold(' Graphics not enabled.')
      endif
      goto 400
c
c <READ> Read model 
c ----------
8000  write(*,1111) ed
      ext = '.mod'
      prmpt = 'input model'
      iwrite = 0
      iform = 1
      ierr = 0
c
      call openfil (lunmod,cmodnam,ext,prmpt,iwrite,iform,ierr)
      if(ierr.eq.1) go to 400 
c
      ierror = 0
      call readmod (ierror)
c
      do 8010 ktype=1,ntypes
        if(nfield(ktype).gt.0.and.nfield(ktype).le.maxobs) then
          do 8005 ipt=1 , nfield(ktype)
8005      xpos(ipt,ktype) = xoffs(ktype) + (ipt-1)*space(ktype)
          xlpl = xpos(1,ktype)
          xupl = xpos(nfield(ktype),ktype)
          zlpl = zmax
          zupl = 0
          mode = 1
        else
          if(nfield(ktype).lt.0.or.nfield(ktype).gt.maxobs) then
            call error(' Too many or zero profile points')
          endif
        endif
8010  continue
      call sav
c
c Clear screen
c
c      call inqvie(xx1,zz1,xx2,zz2)
c      call setvie(0.0,0.0,1.0,1.0,nclr,nclr)
c      call setcol(iblk)
c     call clr
c      call setvie(xx1,zz1,xx2,zz2,nclr,nclr)
      go to 400
c
c <WRIT> Write model
c -----------
9000  write(*,1111) ed
      ext = '.mod'
      prmpt = 'output model'
      iwrite = 1
      iform = 1
      ierr = 0
c
      call openfil (lunmod,cmodnam,ext,prmpt,iwrite,iform,ierr)
      if(ierr.eq.1) go to 400 
c
      ierror = 0
      call writmod(ierror)
c
      goto 400
c
c <ANOM> Calculates Anomalies
c -------------------
10000 if(nbods.eq.0) then
        call bold ('No bodies defined,select option "EBOD"')
        go to 400
      endif
      if(nfield(itype).eq.0) go to 98000
      write(*,*)'Enter body no. for anomaly calculation,'
      write(*,'(a\)')' or "0" for total anomaly : '
      read(*,*,err=99000) ibod
      if(ibod.lt.0.or.ibod.gt.nbods) then
        call error('Body number can be from zero to maximum number') 
        go to 10000
      endif
      if(igrap.ne.0) then
        if(ibod.eq.0) then
          call planom(0,iblk)
        else
          call planom(ibod,nclr)
        endif
        if(idiff.lt.0) call plobs
      else
        if(kalk.eq.1) call calcan
        write(*, *) ' Anomaly calculated'
      endif
      go to 400
c
c <TANO> Prints anomaly
c -----------------
11000 write(*,1111) ed
      write(*,'(a\)')' Enter body no. to type (0 for all) : '
      call sound (20,200)
      read(*,*,err=11000) ibod
        if(ibod.ge.0.and.ibod.le.maxcal) then
          call typano(ibod)
        else
          call error ('Invalid body number, retry')
          go to 11000
      endif
      goto 400
c
c <TOBS> Print out observations
c ----------------------
12000 call typobs
      goto 400
c
c <ECOM> Input comments
c --------------
13000 write(*,1111) ed
      write(*,*) 'Enter comments (up to 80 char) :'
      call sound (20,200)
      read(*,13010,err=99000) kommnt
13010 format(a)
      goto 400
c
c <TCOM> Output comments
c ---------------
14000 write(*,1111) ed
      write(*, 14010) kommnt
14010 format(1h,a)
      go to 400
c
c <PRNT> Print screen to printer
c -----------------------
15000 call prnt
      go to 400
c
c <INSE> Insert point
c --------------------
16000 call inse
      goto 350
c
c <DPOI> Delete point
c ------------
17000 call dele
      goto 350
c
c <END > Exit program
c ------------
18000 write(*,1111) ed
      write(*,*)' Did you use option "WRITe" to store your final'
      write(*,*)' model on disc ?'
      write(*,*) ' '
      write(*,'(a\)')' Enter "Y" to  END  : '
      call sound(20,200)
      read(*,'(a)',err=99000) cans
      if(cans.ne.'y'.and.cans.ne.'Y') go to 400
      write(*,'(/a/)') ' Magrav terminated'
      close(lunscr,status='delete')
      close(model)
      go to 99999
c
c <TABL> To graphics tablet control
c -----------------
19000 if(nfield(itype).eq.0) go to 98000
      if(igrap.ne.0) then
        call grap
      else
        call bold ('Graphics not enabled.')
      endif
      goto 400
c
c  <RECO> Recover previous step
c ---------------------
20000 call reco
      if(igrap.ne.0) then
        call planom (0,iblk)
        call plobs
        call plbod (0,igry)
      endif
      goto 400
c
c <DUMP> Dump current data
c ------------------
21000 call tsca
      if(itype.eq.2) then
        write(*,'(/a)')'     MAGNETIC DUMP'
        write(*,'(a)')'     -------------'
        write(*,21010) dip,dec,xton
21010   format(1h0,'Main field dip :',f6.1,/,
     +' Main field declination :',f6.1,/,
     +' Profile +ve X to North angle:',f6.1,/)
      else
        write(*,'(/a)')'      GRAVITY DUMP'
        write(*,'(a)')'      ------------'
      endif
      call typbod(0)
      call typobs
      ibod = 0
      call typano(ibod)
      goto 400
c
c <MAUT> Move point automatically
c -------------
22000 write(*,1111) ed
      write(*,*)'Select the body containing the point to '
      write(*,'(a\)')' be moved automatically : '
      call sound (20,200)
      read(*,*,err=99000) ibod
      if(npts(ibod).eq.0) then
        call error(' This body number is not defined')
        go to 400
      endif
      call typbod(ibod)
      write(*,'(a\)')' Select point to be moved automatically : '
      call sound (20,200)
      read(*,*,err=99000) npt
      if(npt.lt.1.or.npt.gt.npts(ibod)) then
        call error('This point number not defined in this body')
        go to 400
      endif
      write(*,1111) sgr5
      write(*,*)'Processing .....'
      write(*,1111) sgr0
      call amove(x(npt,ibod),z(npt,ibod),0.0,0.0,2)
      if(igrap.ne.0) then
        call plbod (ibod,nclr)
        call planom (0,nclr)
      endif
      go to 350
c
c <TPAR> Type out body parameters
c ------------------------
23000 call typar
      goto 400
c
c <MAGN> Change to magnetics mode and enter parameters
c ----------------------
24000 write(*,1111) ed
      if(itype.eq.2) then
        write(*,'(/a)')' You are already in MAGNETICS mode'
      else
        write(*,'(/a)')' You are now in MAGNETICS mode'
      endif
      if(itype.gt.0) then
        write(*,'(a\)')' Change the magnetic mode parameters(y/n) : '
        call sound (20,200)
        read(*,'(a)') cans
        if(cans.eq.'n'.or.cans.eq.'N') go to 24050
      endif
      write(*,*)'Enter the angle from geographic North to the '
      write(*,'(a\)')'  positive X (or profile) direction (cw) : '
      call sound (20,200)
      read(*,*,err=99000) xton
      xton = amod(xton,360.)
      if(xton.lt.0.) xton = xton + 360.
      write(*,'(a\)')' Enter magnetic field declination : '
      call sound (20,200)
      read(*,*,err=99000) dec
      dec = amod(dec,360.)
      if(dec.lt.0.) dec = dec + 360.
      write(*,'(a\)')' Enter magnetic field dip : '
      call sound (20,200)
      read(*,*,err=99000) dip
      dip = amod(dip,360.)
      if(dip.lt.0.) dip = dip + 360.
      write(*,'(a\)')' Enter depth offset added to body points(km) : '
      call sound (20,200)
      read(*,*,err=99000) zcon(2)
      if(zcon(2).le.0.) zcon(2) = .001
24050 itype = 2
      call rean
      goto 350
c
c <GRAV> Change to gravity mode and enter "zcon"
c -------------------------
25000 write(*,1111) ed
      if(itype.eq.1) then
        write(*,'(/a)')' You are already in GRAVITY mode.'
      else
        write(*,'(/a)')' You are now in GRAVITY mode.'
      endif
      if(itype.gt.0) then
        write(*,'(a\)')' Change gravity mode parameter(y/n) : '
        call sound (20,200)
        read(*,'(a)') cans
        if(cans.eq.'n'.or.cans.eq.'N') go to 25050
      endif
      write(*,'(a\)')' Enter depth offset added to bodies(km) ; '
      call sound (20,200)
      read(*,*,err=99000) zcon(1)
      if(zcon(1).le.0.) zcon(1) = .001
25050 itype = 1
      call rean
      goto 350
c
c <MENU> Menu of Commands
c -----------------
26000 write(*,1111) ed
      call menopt
      goto 400
c
c <ZOOM> Set limits for "sketch"
c ----
27000 write(*,1111) ed
      if(igrap.eq.0) then
        write(*,*)'Graphics not enabled.'
        go to 400
      endif
      write(*,'(/a\)')' Enter minimum X for zoom (km) : '
      call sound (20,200)
      read(*,*,err=99000) skxmin
      write(*,'(a\)')' Enter maximum X for zoom (km) : '
      call sound (20,200)
      read(*,*,err=99000) skxmax
      write(*,'(a\)')' Enter minimum Z for zoom (km) : '
      call sound (20,200)
27011 read(*,*,err=99000) skzmin
      if(skzmin.lt.0) then
        call error(' Minimum Z must be greater than 0')
        go to 27011
      endif
      write(*,'(a\)')' Enter maximum Z for zoom (km) : '
      call sound (20,200)
      read(*,*,err=99000) skzmax
      if(skxmin.ge.skxmax) then
        call error(' Invalid X zoom coordinates,retry')
        go to 27000
      endif
      if(skzmin.ge.skzmax) then

        call error(' Invalid Z zoom coordinates,retry')
        go to 27000
      endif
      mode = 1
      write(*,*)' Call option "SKET" to plot selected area.'
      goto 350
c
c <MSCA> Manually set scaling
c ----------
28000 call mscale
      goto 400
c
c Not used
c --------
29000 go to 400
c
c <TBOD> Prints out body points
c ------------
30000 write(*,1111) ed
      write(*,'(/a\)')' Enter body number to print("0" = all): '
      call sound (20,200)
      read(*,*,err=99000) ibod
        if(ibod.ge.0.and.ibod.le.maxbod) then
          call typbod(ibod)
        go to 400
        endif
      call error(' Body number must be from zero to maximum mumber')
      goto 30000
c
c <TSCA> Prints out scaling parameters
c ----------
31000 write(*,1111) ed
      call tsca
      goto 400
c
c <OFFS> Recalculate offset
c ------------------
32000 write(*,1111) ed
      if(igrap.ne.0) then
        if(nfield(itype).gt.0) then
          write(*,*)'Old offset = ',offset(itype)
          sum = 0.
          do 32050 i = ostart(itype), oend(itype)
            sum = sum + calc(maxcal,i) - obs(itype,i)
32050     continue
          offset(itype) = sum/nfield(itype)
          write(*,*)'New offset = ',offset(itype)
          write(*,*)'(for manual offset calc. use tablet control)'
          if(igrap.ne.0.and.idiff.eq.0)call plobs
        else
          call bold(' WARNING;Cannot calculate offset,')
          call bold('      no observed data')
        endif
      else
        call bold( '"OFFS" is applicable only in graphics mode')
      endif
      goto 400
c
c <DIFF> Difference plotting on/off
c -----------------
33000 idiff = -idiff
      if(idiff.gt.0) then
        write(*,*)'Difference mode now ON'
        if(igrap.ne.0) then
          call planom(0,iblk)
        endif
      else
        write(*,*)'Difference mode now OFF'
        if(igrap.ne.0) then
          call planom(0,iblk)
          call plobs
        endif
      endif
      goto 400
c
c <HELP> Help text
c ---------
34000 write(*,1111) sgr3047, ed
      write(*,34010)
34010 format(1h ,
     +'  Magrav is a 2.5 dimensional magnetics and gravity modeling',
     +/,' program. Three modes of operation are possible :',
     +/,'   1) Text mode with no graphics',
     +/,'   2) Graphics enabled with keyboard control',
     +/,'   3) Graphics enabled with graphics tablet control',
     +/,' To start modelling the following options are called',
     +/,' in order : MAGN or GRAV,EOBS,EBOD,EPAR,ANOM.',/)
34020 call menopt
      write(*,*)' Additional help is available for each option'
      write(*,'(a\)')' Enter HELP option(<CR> to return) : '
      call what(iwhat)
      if(iwhat.eq.-1) go to 34021
      if(iwhat.eq.0) then
        write(*,1111) ed
        write(*,*)' This option not recognized, try again'
        go to 34020
      endif
      call help(iwhat)
      go to 34020
c
34021 write(*,1111) sgr0
      write(*,1111) ed
      go to 400
c
c <MBOD> Move body
c ----------------
35000 write(*,1111) ed
      write(*,'(a\)')' Enter number of body to be moved : '
      call sound (20,200)
      read(*,*,err=99000) ibod
      if(npts(ibod).eq.0) then
        call error(' This body is not defined')
        go to 400
      endif
      call typbod(ibod)
      write(*,'(a\)')' Enter X shift for body(km) : '
      call sound (20,200)
      read(*,*,err=99000) xshift
      write(*,'(a\)')' Enter Z shift for body(km) : '
      call sound (20,200)
      read(*,*,err=99000) zshift
      do 35010 j = 1 , npts(ibod)
        x(j,ibod) = x(j,ibod) + xshift
        z(j,ibod) = z(j,ibod) + zshift
35010 continue
      if(igrap.ne.0) call plbod (0,igry)
      ianom(ibod) = 0
      kalk = 1
      go to 350
c
c <DBOD> Delete body
c ------------------
36000 write(*,1111) ed
      write(*,'(a\)')' Enter number of body to be deleted : '
      call sound (20,200)
      read(*,*,err=99000) ibod
      if(ibod.lt.1.or.ibod.gt.maxbod) then
        call error(' Body no. must be from 1 to maximum')
        go to 400
      endif
      if(npts(ibod).eq.0) then
        call error(' This body number not defined')
        go to 400
      endif
      call typbod(ibod)
      call delbod (ibod)
      if(igrap.ne.0) then
        call plbod(0,igry)
        call planom(0,nclr)
      endif
      go to 350
c
c <ASCL> Automatic scaling for anomaly plot
c -----------------------------------------
37000 if(igrap.ne.0) then
        call ascale
        call planom(0,iblk)
        if(idiff.lt.0)call plobs
      else
        call bold('"ASCA" is applicable only in graphics mode')
      endif
      go to 400
c
c <CONF> Program configuration
c
38000 call config(1)
      go to 400
c
c <SAVE) Save Halo format image and palette files
c
39000 ierr = 0
      call saveh
      go to 400
c
c Incorrect order branch
c ----------------------
98000 call error(' Before this option can be called ')
      call bold( ' either an model must be "READ" in or')
      call bold( ' observed data be read in or entered')
      call bold( ' manually using option "EOBS"')
      go to 400
c
c input error branch
c ------------------
99000 call error (' Input error , retry ;')
      go to 400
c
99999 call setloc ('//',2)
      call closeg
      call chkerr
      stop
      end
c
c *****************************************************************
      SUBROUTINE ERROR (STRNG)
c
c Purpose : To display bold, blinking error messages on the screen
c           and call the error sound subroutine
c
      character  strng*(*)
c
$include:'magrav.cmn'
c
      write(*,1111) sgr1,sgr5
      write(*,*) ' '
      write(*,'(a\)') ' ERROR :'
      write(*,1111) sgr0, sgr1
      write(*,*) strng
      write(*,1111) sgr0
      call sound(15,6000)
      return
      end
c
c *****************************************************************
      SUBROUTINE BOLD (STRNG)
c
c Purpose : To display a bold message on the screen
c
      character  strng*(*)
$include:'magrav.cmn'
c
      write(*,1111) sgr1
      write(*,*) strng
      write(*,1111) sgr0
c
      return
      end
c
c *****************************************************************
c
      SUBROUTINE MENU(N,TITLE,PROMPT,IOPTION,IERR)
c
c Revision : 1.1, Feb. 22, 1990, Broome
c
c Purpose : General purpose menu routine for FORTRAN programs
c
      character*72 prompt(10),title
c
$include:'menu.cmn'
c
      if(n.gt.10) then
        call error('MENU: number of prompts exceeds limit of 10')
        ierr = 1
        return
      endif
c
c Clear graphics and home cursor
c
100   write(*,1111) sgr0
      write(*,1111) ed
      write(*,1111) sgr7
c
c Print title
c
      write(*,1111) sgr1
      write(*,*) title
      write(*,1111) sgr0
c
      write(*,*)' '
c
c Print menu options
c
      do 200 j=1, n
        write(*,1010) j,prompt(j)
1010    format('     ',i2,') ',A72)
200   continue
c
c Prompt user for selection
c
220   write(*,1111) sgr1
      write(*,'(/a\)') ' Select option by number : '
      write(*,1111) sgr0
      read(*,*,err=400) ioption
      if(ioption.gt.n.or.ioption.le.0) then
        write(*,*) 'Select an option between 1 and ',n
        go to 220
      endif
      go to 999
c
400   write(*,*)' '
      write(*,*)' Input error, retry'
      call hret
      go to 100
c
999   return
      end
c
c *********************************************************
      SUBROUTINE ANSIST
c
c Purpose: to initialize ANSI command strings
c
$include:'magrav.cmn'
c
c ESC : Escape character
      ESC    = char(27)
c
c CSI : ANSI prefix
      CSI    = esc // '['
c
c UL : Underline
      UL     = csi // '4m'
c
c ED : Erase display and home cursor 
      ED    = csi // '2J' 
c
c SGR0 : Reset all graphics attributes
      SGR0    = csi // '0m'
c
c SCP : Save cursor position
      SCP    =  csi // 's'
c
c RCP : Restore cursor position
      RCP    = csi // 'u'
c
c SGR7 : Reverse video
      SGR7    = csi // '7m'
c
c SGR3047 : white background; black foreground
      SGR3047   = csi // '47m' // csi // '30m'
c
c CUU1 : Cursor up one line
      CUU1   = csi // '1A'
c
c CUD1 : Cursor down one line
      CUD1   =  csi // '1B'
c 
c CUF1 : Cursor forward one column
      CUF1   =  csi // '1C'
c
c CUB1 : Cursor back one column
      CUB1   =  csi // '1D'
c
c NL : New line (line feed)
      NL     = esc // 'E'
c
c RET : Nul character
      NUL    = char(0)
c
c EL : Erase current line
      EL = csi // 'K'
c
c SGR5 : blink text
      SGR5 = csi // '5m'
c
c SGR1 : Bold text
      SGR1 = csi // '1m'

c
      return
      end
c
c ***********************************************************
      SUBROUTINE OPENFIL (LUN,NAME,EXT,PROMPT,IWRITE,IFORM,IERR)
c
c Purpose : To open a file used by the program
c
c Parameters : lun    - logical unit number (int*2)
c              name   - 40 character root name for file
c              ext    - 4 character filename extention to be used
c              prompt - 20 character file descriptor for messages
c              iwrite - "1" for open output file, "0" to open 
c                         existing read-only file
C              iform  - "1" to open formatted file, "0" for unform.
c              ierr   - set to one if files not opened
c
      integer*2 lun
      character name*40, ext*4, prompt*20, filnam*44, yesno*10
      character ftype*12, nameo*40
      logical fex
c
      nameo = name
      if(iform.eq.1) then
        ftype = 'formatted'
      elseif (iform.eq.0) then
        ftype = 'unformatted'
      else
        call error ('OPENFIL:"iform" argument invalid')
      endif
c
c Add extention to default name
c
      yesno = 'y'
      if(name.eq.'none') go to 40
      call lcvar (l,name)
      filnam = name
      filnam(l:l+4) = ext
      go to 30
c
29    write(*,*)'ERROR, opening file, retry '
30    write(*,*) ' '
      write(*,*)'Default ', prompt,' file name: ',filnam
      write(*,'(a\)')' Change this name? (y/n)[n]: '
      read(*,'(a)') yesno
c
c Change root name of file
c
40    if(yesno.eq.'N'.or. yesno.eq.' ') yesno= 'n'
      if(yesno.ne.'n') then
        write(*,1000) prompt, ext
1000    format(/,' Enter root name for ',a20,' file,',/,
     +  ' ("',a4,'" will be added)("esc" to quit): ',\)
        read(*,'(a)') name
        if(name.eq.'esc'.or.name.eq.'ESC') then
          name = nameo
          go to 990
        endif
        call lcvar(l,name)
        filnam = name
        filnam(l:l+4) = ext
      endif
c
c Check if files already exist, then if it is an output file
c   should it be overwritten
c
      inquire(file=filnam,exist=fex)
      if(iwrite.eq.1) then
        if(fex.eqv..true.) then
          write(*,*) '  '
          write(*,*)'ERROR, file ',filnam,' already exists'
          write(*,'(/a\)') ' Overwrite the file? (y/n)[n]: '
          read(*,'(a)') yesno
          if(yesno.eq.'Y'.or.yesno.eq.'y') then
            close (lun)
            open(lun,file=filnam,status='old',form=ftype,err=29)
            go to 999
          else
            yesno = 'y'
            go to 40
          endif
        endif
        close (lun)
        open(lun,file=filnam,status='new',form=ftype,err=29)
c
c If the file is a read only file:
c
      else
        if(fex.eqv..true.) then
          close (lun)
          open(lun,file=filnam,status='old',form=ftype,err=29)
        else
          call error('File not found')
          yesno = 'y'
          go to 40
        endif
      endif
c
      go to 999
c
c Set error flag if files not opened
c
900   call error ('File not opened')
c
990   ierr = 1
c
999   return
      end      
