c
c ************************************************************
c
c Program : PROPLT
c
c Revision : 2.0; SEPT. 7, 1988
c
c Purpose : To plot profile data on the Epson FX-series 
c            printers.
c           NOTE: Epson printer must be in Epson mode, not
c                  IBM-compatible mode
c
c Files : 7 - input coded data file
c        11 - lpt1 (Epson printer)
c
c Author  : John Broome
c           Potential Fields Section
c           Geological Survey of Canada
c           Ottawa, Canada
c
c        **  Crown Copywrite **
c
c Part of Geological Survey of Canada Open Files 1334 and
c  1581
c
c ***********************************************************
c
      program proplt
c
      integer*2 bitshft(8),iclr(100)
      integer*4 ix(3000),iy(3000)
      real*4 data(3000)
      character pout(1000),pdat(1000),esc
      character*40 filspec*40, ans*40, dir*20, ed*4, title*80
      character*11 clinum,cpscl,cgspac,czscl,cnpts,cspace
      logical*2 fex
      equivalence (pdat(1),pout(6))
c
      data bitshft/128,64,32,16,8,4,2,1/
c
      ed = char(27) // '[2J'     
      write(*,1111) ed
1111  format (1x,a,\)
      write(*,*)' '
      write(*,*)'       ----------------------------------'      
      write(*,*)'       |                                |'
      write(*,*)'       |  *******  PROPLT 2.0  *******  |'
      write(*,*)'       |                                |'  
      write(*,*)'       ----------------------------------'
      write(*,*) ' '
      write(*,*) '  PROPLT is part of the Geological Survey of Canada'
      write(*,*) ' Potential field workstation software package.'
      write(*,*) ' '
      write(*,*) '  PROPLT plots MAGRAV2-compatible profile'
      write(*,*) ' files on Epson FX-series dot-matrix printers.'
      write(*,*) ' (Note: Your printer must be in Epson mode)'
c
c Enter boundary value flag
c
      write(*,'(/a\)')' Enter no-data flag (-9999.0) : '
      call sound (20,200)
      read(*,*) bndval
c
10    write(*,'(/a\)')' Enter profile file name("esc"to quit): '
      call sound (20,200)
      read(*,'(a)') filspec
      if(filspec.eq.'esc'.or.filspec.eq.'ESC') go to 9999
      open(unit=7,file=filspec,status='old',form='formatted')
c
      inquire (file=filspec,exist=fex)
      if(fex.eqv..false.)then
        call error ( 'Profile file not found')
        call hret
        write(*,1111) ed
        go to 10
      endif
      open(unit=11,file='lpt1',form='binary')
c
c Read through data to find x and z ranges
c
      rewind (7)
      read(7,*) npts
      read(7,*) space
c
       if (npts.gt.3000) Then
         write(*,1111) ed
         call error ('Maximum number of data points is 3000')
         stop
      endif
c
      datmin = 99999999.0
      datmax = -9999999.0
      zsum = 0
      ndat = 0
c
c Initialize arrays
c
      do 40 j = 1 ,1000
        pdat(j) = char (0)
40    continue
c
      do 100 ipt = 1 ,npts
90      if (ierr.ne.0) then
          write(*,1111) ed
          call error (' ')
          write(*,1055) ierr,ipt
1055      format(1h0,'Error ',i6,' reading record ',i10)
        endif
c
        read(7,*,err=90) data(ipt)
        if (data(ipt).eq.bndval) go to 100
        ndat = ndat + 1
        datmin = amin1(data(ipt),datmin)
        datmax = amax1(data(ipt),datmax)
        zsum = zsum + data(ipt)
100   continue
c
      rewind (7)
c
      dz = datmax-datmin
      zavg = zsum/ndat
      write (*,1111) ed
      write(*,1065) filspec, npts, datmin, datmax, dz, zavg
1065  format(/,  ' Profile file name ...',a,/,
     +           ' Number of points ....',i11,/,
     +           ' Minimum data value ..',f11.4,/,
     +           ' Maximum data value ..',f11.4,/,
     +           ' Data range ..........',f11.4,/,
     +           ' Average value .......',f11.4,/)
c
      write(*,'(/a\)')' Enter spacing for grid overlay (cm): '
      call sound (20,200)
      read(*,*) gspac
c
      write(*,'(/a\)')' Enter data scale (units/cm.) ......: '
      call sound (20,200)
      Read(*,*) zscl
c
      write(*,'(/a)')' Enter output profile scale,'
      write(*,'(a\)')'  (enter "10000" for 1:10000 scale) : '
      call sound (20,200)
      read(*,*) pscl
c
      write(*,'(/a)')' Enter text string to print on plot .:'
      call sound (20,200)
      read(*,'(a)') title
c
      gspac = gspac * zscl
c
c Calculate the value to be assigned to the page center
c  and scaling factors
c
      pspace = space * 100000 * 28.34 / pscl
      scld = 72.0 / 2.54 / zscl
      gsmin = ifix (datmin/gspac - 1.0) * gspac
      maxgrd = ifix ( ( dz + 2*gspac ) *scld)
c
c Fill the header portion of the print array 
c
      esc = char(27)
      pout(1) = esc
      pout(2) = '*'
      pout(3) = char(5)
c
c Write profile parameters to profil
c
      write (cnpts,'(i11)')    npts
      write (cspace,'(f11.4)') space
      write (cpscl,'(f11.2)')  pscl
      write (czscl,'(f11.4)')  zscl
      write (cgspac,'(f11.4)') gspac
c
      write(11) ' ',title,char(10),char(13),char(10),char(13)
      write(11) ' Profile file name   :',filspec,char(10),char(13)
      write(11) ' Number of points    : ',cnpts,char(10),char(13)
      write(11) ' Point spacing(km)   :',cspace,char(10),char(13)
      write(11) ' Plot scale          : ',cpscl,char(10),char(13)
      write(11) ' data scale(units/cm): ',czscl,char(10),char(13)
      write(11) ' grid spacing (units):', cgspac, char(10),     
     + char(13)
      write(11) char(10),char(10),char(10),char(10),char(13)
c
c Program the printer for plotter graphics
c
      write(11) esc,'@'
      write(11) esc,'A',char(8)
c
c Setup the plot grid
c
      pdat(1) = char(255)
      do 105 igrd = 1 , 100
        gval = igrd*gspac*scld
        kxg = ifix(gval)
        if (kxg.gt.maxgrd) go to 106
        pdat(kxg) = char (255)
105   continue
c
c Read profil data and plot on printer
c
106   npos = 0
      maxp = 0
      nytot = 0
      ncl = 0
      iflag = 1
      ip = 1
      if (data(ip).eq.bndval) then
        zz = zavg
      else
        zz = data(ip)
      endif
      z2 = (zz-gsmin)*scld
      go to 110
c
108   z1  = z2
c
110   ip = ip + 1
c      write(*,*) 'ip,data(ip)=',ip, data(ip)
      if (ip.gt.npts) go to 999
      z1 = z2
      if (data(ip).eq.bndval) then
        zz = zavg
      else
        zz = data(ip)
      endif
      z2 = (zz-gsmin) * scld
      ix1 = ix2
      ix2 = ifix(pspace*ip + 0.5)
      nyl = 0
c
      zd  =  (z2-z1)/pspace
c
120   if (npos.ge.8) go to 200
      nyl = nyl + 1
      if (nytot.gt.(pspace*(ip-1))) then
        iflag = 1
        go to 108
      endif
      nytot = nytot + 1
      npos = npos + 1
c
c Plot data  profile point
c
      kxz = ifix (z1 + nyl*zd )
      if (ichar(pdat(kxz)).ne.255) then
        pdat(kxz) = char(ichar(pdat(kxz)) + bitshft(npos))
        maxp = max0(maxp,kxz)
        ncl = ncl + 1
        iclr(ncl) = kxz
      endif
c
c Mark profile point position at edge of plot
c
      if(iflag.eq.1) then
        pdat(2) = char(ichar(pdat(2)) + bitshft(npos))
        pdat(3) = char(ichar(pdat(3)) + bitshft(npos))
        pdat(4) = char(ichar(pdat(4)) + bitshft(npos))
        do 140 k = 2 , 4
          ncl = ncl + 1
          iclr(ncl) = k
140     continue
        iflag = 0
      endif
      go to 120
c
c Write data to printer
c (set unidirectional printing
c
200   pout(4) = char(mod(kxg,256))
      pout(5) = char(kxg/256)
      nout = kxg + 5
c
      write (11) esc,'<'
      write (11) (pout(j),j=1,nout)
      write (11) char(10)
c
      npos = 0
      maxp = 0
c
      do 300 j = 1 , ncl
300   pdat(iclr(j)) = char(0)
c
      ncl = 0
      go to 120
c
c Print out profil values if required
c
999   write(*,'(/a\)')' Print profil values ? (y/n)  : '
      read(*,'(a)') ans
      if(ans.eq.'Y'.or.ans.eq.'y') then
        write(11) esc,'@'
        close (11)
        open(unit=11,file='lpt1')
        write(11,*)' '
        write(11,*)' Profile values :'
        write(11,*)' --------------'
        do 400 j = 1 , npts
          write(11,1110) j,data(j)
1110      format(i4,') ',f10.2)
400     continue
        close (11)
        open(unit=11,file='lpt1',form='binary')
      endif
c
      write(11) char(12)
      write(11) esc,'@'
      close (11)
      close (7)
      write(*,1111) ed
      go to 10
c
9999  stop
      end
c
c ******************************************************************
      SUBROUTINE HRET
c
c Purpose: This routine tells user to hit a key to continue
c
      character*1 dum
      integer     iocheck
c
      write (*,10)
10    format (/, 1x, 'Hit ENTER to continue. ', \)
      read (*, '(a)', iostat=iocheck) dum
c
      END
c
c *****************************************************************
      SUBROUTINE ERROR (STRING)
c
c Purpose : To display bold, blinking error messages on the screen
c           and call the error sound subroutine
c
      character  string*(*), esc*1, csi*2, sgr5*4, sgr1*4
      character  sgr0*4
c
      ESC    = char(27)
      CSI    = esc // '['
      SGR5   = csi // '5m'
      SGR1   = csi // '1m'
      SGR0   = csi // '0m'
c
      write(*,1111) sgr1,sgr5
1111  format(1x,a,\)
      write(*,*) ' '
      write(*,'(a\)') ' ERROR :'
      write(*,1111) sgr0, sgr1
      write(*,*) string
      write(*,1111) sgr0
      call sound(15,6000)
      return
      end
